package PureMessage::Multifile::DB;
use strict;
use warnings;
use Digest::MD5 qw/md5_hex/;
use PureMessage::Version qw/$PREFIX/;
use File::Path qw/mkpath/;
use PureMessage::Group::PermissionItem;
use PureMessage::Multifile::DB::GroupContent;
use base 'PureMessage::DBI';

__PACKAGE__->table('multifile');
__PACKAGE__->columns(Primary => 'multifile_id');
__PACKAGE__->columns(All => qw/multifile_id charset name type/);
__PACKAGE__->has_many(group_files => 'PureMessage::Multifile::DB::GroupContent');

my $VALID_CHARSETS = qr/^(?:latin1|big5|big5hkscs|gb2312|ksc5601|euckr|iso2022jp)$/;
my %VALID_TYPES = (
    banner   => 1,
    document => 1,
    notification => 1
);

sub add {
    my ($class, %opts) = @_;

    my $id = lc($opts{id});
    die "Invalid id: $id\n" unless $id =~ /^[a-z][\w.-]+$/;
    die "Name is mandatory!\n" unless $opts{name};
    # XXX Disable charset validation to make pmx_add_banner work
    # die "Invalid charset: $opts{charset}\n" 
    #     if $opts{charset} and $opts{charset} !~ $VALID_CHARSETS;
    my $type = $opts{type} or die "Type is mandatory!\n";
    die "Invalid multifile type: $type\n" unless $VALID_TYPES{$type};

    my $existing = PureMessage::Multifile::DB->search(multifile_id => $id)->first;
    die "'$id' already exists!\n" if $existing;

    my $mf = $class->create({ multifile_id => $id,
                              name         => $opts{name},
                              type         => $type,
                              ($opts{charset} ? (charset => $opts{charset}) : ()),
                            });
    $mf->update;

    # Add corresponding permission
    my $perm = "configuration.${type}s.$id";
    PureMessage::Group::PermissionItem::ReadWrite->add_permission_item($perm);
    return $mf;
}

sub delete_file {
    my ($class, $id) = @_;
    my $mf = $class->search(multifile_id => lc($id))->first;
    die "Multifile $id does not exist!\n" unless $mf;
    my $type = $mf->type;
    $mf->delete;

    eval { # ignore errors
        my $perm = "configuration.${type}s.$id";
        PureMessage::Group::PermissionItem::ReadWrite->delete_permission_item($perm);
    };
    warn $@ if $@;
}

sub load {
    my ($class, $id) = @_;
    $class->clear_object_index();
    return $class->search(multifile_id => $id)->first;
}

sub set_content {
    my ($self, $group, $content) = @_;
    die 'content is mandatory' unless defined $content;

    my $gmf = $self->_fetch_group_content($group, 1);
    my $class = ref $gmf;
    if (length $content) {
        $gmf->content($content);
        $gmf->update;
    }
    else {
        $gmf->delete;
    }
}

sub content {
    my ($self, $group) = @_;
    my $gmf = $self->_fetch_group_content($group);
    return $gmf ? $gmf->content : '';
}

sub _fetch_group_content {
    my ($self, $group, $create) = @_;
    die 'group is mandatory' unless $group;
    my %opts = (
        multifile_id => $self,
        group_name => $group,
    );
    my $gc = PureMessage::Multifile::DB::GroupContent->search(%opts)->first;
    if (!$gc and $create) {
        $gc = PureMessage::Multifile::DB::GroupContent->create(\%opts);
    }
    return $gc;
}

sub should_sync {
    my $self = shift;
    return defined $VALID_TYPES{$self->type};
}

1;
